#include <iostream>
#include <cmath>
#include "Infiltration_depth.h"
#include "Parameters.h"
using namespace std;
//Gathers parameters from the class "Parameters"
Parameters par; //Gathers the parameters from the class "Parameters"
float Yb = par.getYb();
float Os = par.getOs();
float lambda = par.getlambda();
float ORi = par.getORi();
float Ks = par.getKs();
float Hc = par.getHc();
float timestep = par.getTimestep();
float k1,k2,k3,k4;
//Calculates the advance in Z direction for a wetting front k > 1
float Infiltration_depth::setZKn(float K0, float K0L, float O0, float O0L, float Z_, float OR0, float OR0L){
    float G;        //initialize G
    if (O0 < Os){   //Unsaturated G
        G = Yb*(pow(OR0,3+1/lambda)-pow(OR0L,3+1/lambda))/(3*lambda+1);
    }else{          //Saturated G
        G = Yb*(3*lambda+2-pow(OR0L,3+1/lambda))/(3*lambda+1);
    }               //RK4 calculations of k1-k4
    k1 = timestep*((K0-K0L)/(O0-O0L))*((G/Z_)+1);
    k2 = timestep*((K0-K0L)/(O0-O0L))*((G/(Z_+k1*0.5))+1);
    k3 = timestep*((K0-K0L)/(O0-O0L))*((G/(Z_+k2*0.5))+1);
    k4 = timestep*((K0-K0L)/(O0-O0L))*((G/(Z_+k3))+1);
    ZKn = Z_+(k1+2*k2+2*k3+k4)/6;   //Adds delta Z to Z
}
//Infliltration Depth for wettingfront k = 1;
float Infiltration_depth::setZK1(float K0, float O0, float O0L, float Z_, float OR0, float OR0L){
    float G;
    if (O0 < Os){
        G = Yb*(pow(OR0,3+1/lambda)-pow(OR0L,3+1/lambda))/(3*lambda+1);
    }else{
        G = Yb*(3*lambda+2-pow(OR0L,3+1/lambda))/(3*lambda+1);
    }               //RK4 calculations of k1-k4
    k1 = timestep*((K0/(O0-O0L))+(Ks*G/((O0-O0L)*Z_)));
    k2 = timestep*((K0/(O0-O0L))+(Ks*G/((O0-O0L)*(Z_+k1*0.5))));
    k3 = timestep*((K0/(O0-O0L))+(Ks*G/((O0-O0L)*(Z_+k2*0.5))));
    k4 = timestep*((K0/(O0-O0L))+(Ks*G/((O0-O0L)*(Z_+k3))));
    ZK1 = Z_+(k1+2*k2+2*k3+k4)/6;  //Adds delta Z to Z
}
//Infliltration Depth for a saturated wettingfront;
float Infiltration_depth::setZs(float O0L, float Z_){
    //RK4 calculations of k1-k4
    k1 = timestep*(Ks/((Os-O0L))*(Hc/Z_+1));
    k2 = timestep*((Ks/(Os-O0L))*(Hc/(Z_+k1*0.5)+1));
    k3 = timestep*((Ks/(Os-O0L))*(Hc/(Z_+k2*0.5)+1));
    k4 = timestep*((Ks/(Os-O0L))*(Hc/(Z_+k3)+1));
    Zs = Z_+(k1+2*k2+2*k3+k4)/6;  //Adds delta Z to Z
}
//Infiltration Depth for first advance for a new wetting front;
float Infiltration_depth::setZK0(float O0L, float OR0L){
    float tau = (timestep*Ks/(Os-O0L));
    float G = (Yb/lambda)*((3*lambda+2)-pow(OR0L,(3+(1/lambda))))/(3+(1/lambda)); // equation 13 GARTO
    ZK0 = 0.5*(tau+sqrt(pow(tau,2)+4*tau*G));
}
float Infiltration_depth::getZKn(){
    return ZKn;
}
float Infiltration_depth::getZK1(){
    return ZK1;
}
float Infiltration_depth::getZK0(){
    return ZK0;
}
float Infiltration_depth::getZs(){
    return Zs;
}

